package com.care.client.service;

import cn.hutool.json.JSONObject;
import com.care.client.config.PlatformPinanbaoConfig;
import com.care.client.config.WxConfig;
import com.care.client.vo.MemberInfoVO;
import com.care.client.vo.MemberInitParams;
import com.care.common.cache.RedisKeyConstant;
import com.care.common.cache.RedisUtil;
import com.care.common.entity.CareMemberInfo;
import com.care.common.exception.BDException;
import com.care.common.service.CareMemberInfoService;
import com.care.common.util.CheckUtils;
import com.care.common.util.HttpUtil;
import com.care.common.util.Result;
import com.care.common.util.WxUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.MessageFormat;

@Slf4j
@Service
public class PassportService extends AbstractPassportService {

    @Resource
    private RedisUtil redisUtil;
    @Resource
    private WxPassportService wxPassportService;
    @Resource
    private CareMemberInfoService careMemberInfoService;
    @Resource
    private PlatformPinanbaoConfig platformPinanbaoConfig;


    public Result<Object> login(MemberInitParams params) {

        if ("wx".equalsIgnoreCase(params.getLoginType())) {
            CheckUtils.valueIsNull(params, "code");
            Result<Object>  result = wxPassportService.login(params);
            return result;
        } else if ("pwd".equalsIgnoreCase(params.getLoginType())) {
            CheckUtils.valueIsNull(params, "phone");
            CheckUtils.valueIsNull(params, "password");
            CareMemberInfo careMemberInfo = careMemberInfoService.detailByPhone(params.getPhone());
            if (careMemberInfo == null){
                return  Result.error("手机号没注册");
            }
            if (!DigestUtils.md5Hex(params.getPassword()).equals(careMemberInfo.getPassword())) {
                return  Result.error("密码错误");
            }
            cache(params);
            return Result.success();
        }
        return Result.error();
    }

    public void logout(String token) {
        CheckUtils.notNull(token, "token");
        MemberInfoVO memberInfoVO = (MemberInfoVO) redisUtil.hget(RedisKeyConstant.PINANBAO_LOGIN_INFO, token);
        if (memberInfoVO != null) {
            redisUtil.hset(RedisKeyConstant.PINANBAO_LOGIN_INFO, token, null);
        }
    }

    public MemberInfoVO checkToken(String token) {
        CheckUtils.notNull(token, "token");
        if (!redisUtil.hHasKey(RedisKeyConstant.PINANBAO_LOGIN_INFO, token)) {
            throw new BDException(400002, "TOKEN WAS WRONG :[{"+token+"}]" );
        }
        return (MemberInfoVO) redisUtil.hget(RedisKeyConstant.PINANBAO_LOGIN_INFO, token);
    }


     public Boolean changePwd(String password,Long memberId) {
         CareMemberInfo careMemberInfo = careMemberInfoService.getById(memberId);
         careMemberInfo.setPassword(DigestUtils.md5Hex(password));
         return careMemberInfoService.updateById(careMemberInfo);
     }

     public Boolean decodePhoneNumber(String code, String encryptedData,String iv) {
         MemberInitParams params = (MemberInitParams) redisUtil.hget(RedisKeyConstant.PINANBAO_LOGIN_INFO, code);
         String sessionKey = params.getSessionKey();
         String openId = params.getOpenid();

         String phone = WxUtil.getPhoneNumber(encryptedData,sessionKey,iv);
        if (StringUtils.isEmpty(phone)) {
            return false;
        }
         CareMemberInfo member = careMemberInfoService.detailByPhone(phone);
         if (member == null) { //注册新用户
             CareMemberInfo careMemberInfo = new CareMemberInfo();
             careMemberInfo.setPhone(phone);
             careMemberInfo.setOpenId(openId);

             String accessToken= wxPassportService.accessToken();
             WxConfig wxConfig = platformPinanbaoConfig.wxConfig();

             String infoUrl = MessageFormat.format(wxConfig.getInfoUrl(),accessToken, openId);
             JSONObject jsonObject = HttpUtil.httpGet(infoUrl);
             log.warn("-----jsonObject:[{}]", jsonObject);
             if(jsonObject != null){
                 String headimgurl = jsonObject.getStr("headimgurl");
                 careMemberInfo.setProfile(headimgurl);
                 String nickname = jsonObject.getStr("nickname");
                 careMemberInfo.setNickname(nickname);
                 int sex = jsonObject.getInt("sex");
                 careMemberInfo.setGender(sex == 1 ? "M" : (sex == 2 ? "W": null));
             }

             boolean flag = careMemberInfoService.save(careMemberInfo);
             if (flag) {
                 cache(params);
                 return true;
             } else {
                 return false;
             }
         } else { //已经注册了
             cache(params);
             return true;
         }
     }
}
