package com.care.client.service;

import cn.hutool.json.JSONObject;

import com.care.client.config.PlatformPinanbaoConfig;
import com.care.client.config.WxConfig;
import com.care.client.vo.MemberInitParams;
import com.care.common.cache.RedisKeyConstant;
import com.care.common.cache.RedisUtil;
import com.care.common.util.HttpUtil;
import com.care.common.util.Result;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.MessageFormat;

@Slf4j
@Service
public class WxPassportService extends AbstractPassportService {

    @Resource
    private RedisUtil redisUtil;

    @Resource
    private PlatformPinanbaoConfig platformPinanbaoConfig;


    public Result<Object> login(MemberInitParams params) {
        WxConfig wxConfig = platformPinanbaoConfig.wxConfig();
        // 解析codeUrl
        String codeUrl = MessageFormat.format(wxConfig.getCodeUrl(), wxConfig.getAppId(), wxConfig.getSecret(), params.getCode());
        // 通过codeUrl获取openid
        JSONObject jsonObject = HttpUtil.httpGet(codeUrl);
        log.warn("-----jsonObject:[{}]", jsonObject);

        String openid = jsonObject.getStr("openid");
        String sessionKey = jsonObject.getStr("session_key");
        log.warn("-----openid:[{}], session_key:[{}]", openid, sessionKey);

        MemberInitParams memberInitParams = new MemberInitParams();
        params.setLoginType("wx");
        memberInitParams.setOpenid(openid);
        memberInitParams.setSessionKey(sessionKey);
        redisUtil.hset(RedisKeyConstant.PINANBAO_LOGIN_INFO, params.getCode(), memberInitParams, RedisKeyConstant.WX_ACCESS_TOKEN_TIME);
        return Result.success();
    }


    public String accessToken() {
        WxConfig wxConfig = platformPinanbaoConfig.wxConfig();
        String access_token = (String) redisUtil.hget(RedisKeyConstant.WX_ACCESS_TOKEN, wxConfig.getAppId());
        if (access_token != null) {
            return access_token;
        }
        return this.flushWxToken();
    }

    private String flushWxToken() {
        WxConfig wxConfig = platformPinanbaoConfig.wxConfig();
        // 解析codeUrl
        String tokenUrl = MessageFormat.format(wxConfig.getTokenUrl(), wxConfig.getAppId(), wxConfig.getSecret());
        // 通过codeUrl获取openid
        JSONObject jsonObject = HttpUtil.httpGet(tokenUrl);
        String access_token = jsonObject.getStr("access_token");
        redisUtil.hset(RedisKeyConstant.WX_ACCESS_TOKEN, wxConfig.getAppId(), access_token, RedisKeyConstant.WX_ACCESS_TOKEN_TIME);
        return access_token;
    }

}
